@@ -453,8 +453,11 @@ void  CtClipboard::_on_clip_data_get(Gtk::SelectionData& selection_data, CtClipb
 // From Clipboard to Plain Text
 void CtClipboard::_on_received_to_plain_text(const Gtk::SelectionData& selection_data, Gtk::TextView* pTextView, bool force_plain_text)
 {
-    Glib::ustring plain_text = selection_data.get_text();
-    CtStrUtil::convert_from_bom(plain_text);
+    Glib::ustring plain_text = selection_data.get_text(); // returns UTF-8 string if text type recognised and could be converted to UTF-8; empty otherwise
+    if (plain_text.empty()) {
+        const std::string rawData = selection_data.get_data_as_string();
+        plain_text = CtStrUtil::convert_raw_to_utf8(rawData);
+    }
     plain_text = str::sanitize_bad_symbols(plain_text);
 
     if (plain_text.empty())
@@ -617,16 +620,17 @@ void CtClipboard::_on_received_to_table(const Gtk::SelectionData& selection_data
 // From Clipboard to HTML Text
 void CtClipboard::_on_received_to_html(const Gtk::SelectionData& selection_data, Gtk::TextView* pTextView, bool)
 {
-    Glib::ustring html_content = selection_data.get_data_as_string();
-    CtStrUtil::convert_from_bom(html_content);
-
+    Glib::ustring html_content = selection_data.get_text(); // returns UTF-8 string if text type recognised and could be converted to UTF-8; empty otherwise
+    if (html_content.empty()) {
+        const std::string rawData = selection_data.get_data_as_string();
 #ifdef _WIN32
-    html_content = Win32HtmlFormat().convert_from_ms_clipboard(html_content);
+        html_content = Win32HtmlFormat().convert_from_ms_clipboard(rawData);
+#else
+        html_content = CtStrUtil::convert_raw_to_utf8(rawData);
 #endif
-
+    }
     html_content = str::sanitize_bad_symbols(html_content);
 
-
     CtHtml2Xml parser(_pCtMainWin->get_ct_config());
     parser.feed(html_content);
     from_xml_string_to_buffer(pTextView->get_buffer(), parser.to_string());
@@ -645,8 +649,11 @@ void CtClipboard::_on_received_to_image(const Gtk::SelectionData& selection_data
 // From Clipboard to URI list
 void CtClipboard::_on_received_to_uri_list(const Gtk::SelectionData& selection_data, Gtk::TextView* pTextView, bool)
 {
-    Glib::ustring uri_content = selection_data.get_text();
-    CtStrUtil::convert_from_bom(uri_content);
+    Glib::ustring uri_content = selection_data.get_text(); // returns UTF-8 string if text type recognised and could be converted to UTF-8; empty otherwise
+    if (uri_content.empty()) {
+        const std::string rawData = selection_data.get_data_as_string();
+        uri_content = CtStrUtil::convert_raw_to_utf8(rawData);
+    }
     uri_content = str::sanitize_bad_symbols(uri_content);
 
     if (_pCtMainWin->curr_tree_iter().get_node_syntax_highlighting() != CtConst::RICH_TEXT_ID)
