@@ -174,6 +174,8 @@ void CtConfig::_populate_keyfile_from_data()
         const CtRecentDocsRestore::iterator mapIt = recentDocsRestore.find(filepath.string());
         if (mapIt != recentDocsRestore.end())
         {
+            snprintf(_tempKey, _maxTempKeySize, "visit_%d", i);
+            _uKeyFile->set_string(_currentGroup, _tempKey, mapIt->second.visited_nodes);
             snprintf(_tempKey, _maxTempKeySize, "expcol_%d", i);
             _uKeyFile->set_string(_currentGroup, _tempKey, mapIt->second.exp_coll_str);
             snprintf(_tempKey, _maxTempKeySize, "nodep_%d", i);
@@ -286,7 +288,7 @@ void CtConfig::_populate_keyfile_from_data()
     _currentGroup = "table";
     _uKeyFile->set_integer(_currentGroup, "table_rows", tableRows);
     _uKeyFile->set_integer(_currentGroup, "table_columns", tableColumns);
-    _uKeyFile->set_integer(_currentGroup, "table_col_width", tableColWidth);
+    _uKeyFile->set_integer(_currentGroup, "table_col_width", tableColWidthDefault);
 
     // [fonts]
     _currentGroup = "fonts";
@@ -378,6 +380,8 @@ void CtConfig::_populate_data_from_keyfile()
             snprintf(_tempKey, _maxTempKeySize, "nodep_%d", i);
             if (_populate_string_from_keyfile(_tempKey, &recentDocRestore.node_path))
             {
+                snprintf(_tempKey, _maxTempKeySize, "visit_%d", i);
+                _populate_string_from_keyfile(_tempKey, &recentDocRestore.visited_nodes);
                 snprintf(_tempKey, _maxTempKeySize, "expcol_%d", i);
                 _populate_string_from_keyfile(_tempKey, &recentDocRestore.exp_coll_str);
                 snprintf(_tempKey, _maxTempKeySize, "curs_%d", i);
@@ -518,17 +522,25 @@ void CtConfig::_populate_data_from_keyfile()
     _populate_string_from_keyfile("special_chars", &specialChars);
     _populate_string_from_keyfile("selword_chars", &selwordChars);
     _populate_string_from_keyfile("chars_listbul", &charsListbul);
+    if (charsListbul.size() < CtConst::CHARS_LISTBUL_DEFAULT.size()) {
+        charsListbul = CtConst::CHARS_LISTBUL_DEFAULT;
+    }
     _populate_string_from_keyfile("chars_toc", &charsToc);
-
-    // bug fix for alfa version when charsTodo is really long
-    Glib::ustring tempCharsToDo;
-    _populate_string_from_keyfile("chars_todo", &tempCharsToDo);
-    if (tempCharsToDo.size() > 4 && tempCharsToDo[0] == tempCharsToDo[3])
-        tempCharsToDo = CtConst::CHARS_TODO_DEFAULT;
-    charsTodo = tempCharsToDo;
-
+    if (charsToc.size() < CtConst::CHARS_TOC_DEFAULT.size()) {
+        charsToc = CtConst::CHARS_TOC_DEFAULT;
+    }
+    _populate_string_from_keyfile("chars_todo", &charsTodo);
+    if (charsTodo.size() != CtConst::CHARS_TODO_DEFAULT.size()) {
+        charsTodo = CtConst::CHARS_TODO_DEFAULT;
+    }
     _populate_string_from_keyfile("chars_smart_dquote", &chars_smart_dquote);
+    if (chars_smart_dquote.size() != CtConst::CHARS_SMART_DQUOTE_DEFAULT.size()) {
+        chars_smart_dquote = CtConst::CHARS_SMART_DQUOTE_DEFAULT;
+    }
     _populate_string_from_keyfile("chars_smart_squote", &chars_smart_squote);
+    if (chars_smart_squote.size() != CtConst::CHARS_SMART_SQUOTE_DEFAULT.size()) {
+        chars_smart_squote = CtConst::CHARS_SMART_SQUOTE_DEFAULT;
+    }
     _populate_string_from_keyfile("latest_tag_prop", &latestTagProp);
     _populate_string_from_keyfile("latest_tag_val", &latestTagVal);
     _populate_string_from_keyfile("timestamp_format", &timestampFormat);
@@ -555,7 +567,7 @@ void CtConfig::_populate_data_from_keyfile()
     _currentGroup = "table";
     _populate_int_from_keyfile("table_rows", &tableRows);
     _populate_int_from_keyfile("table_columns", &tableColumns);
-    _populate_int_from_keyfile("table_col_width", &tableColWidth);
+    _populate_int_from_keyfile("table_col_width", &tableColWidthDefault);
 
     // [fonts]
     _currentGroup = "fonts";
