@@ -88,7 +88,7 @@ CtMainWin::CtMainWin(bool             no_gui,
     _pRecentDocsSubmenu = CtMenu::find_menu_item(_pMenuBar, "RecentDocsMenu");
     _pSpecialCharsSubmenu = CtMenu::find_menu_item(_pMenuBar, "SpecialCharsMenu");
     _pMenuBar->show_all();
-    gtk_window_add_accel_group(GTK_WINDOW(gobj()), _uCtMenu->default_accel_group());
+    add_accel_group(_uCtMenu->get_accel_group());
     _pToolbars = _uCtMenu->build_toolbars(_pRecentDocsMenuToolButton);
 
     _vboxMain.pack_start(*_pMenuBar, false, false);
@@ -770,7 +770,7 @@ void CtMainWin::window_header_update()
         gint64 curr_node = curr_tree_iter().get_node_id();
         int button_idx = 0;
         auto buttons = _ctWinHeader.buttonBox.get_children();
-        auto nodes = get_state_machine().get_visited_nodes_list();
+        auto nodes = _ctStateMachine.get_visited_nodes_list();
         _ctWinHeader.button_to_node_id.clear();
         for (auto iter = nodes.rbegin(); iter != nodes.rend(); ++iter)
         {
@@ -944,6 +944,7 @@ void CtMainWin::_ensure_curr_doc_in_recent_docs()
     {
         _pCtConfig->recentDocsFilepaths.move_or_push_front(fs::canonical(currDocFilePath));
         CtRecentDocRestore prevDocRestore;
+        prevDocRestore.visited_nodes = str::join_numbers(_ctStateMachine.get_visited_nodes_list(), ",");
         prevDocRestore.exp_coll_str = _uCtTreestore->treeview_get_tree_expanded_collapsed_string(*_uCtTreeview);
         const CtTreeIter prevTreeIter = curr_tree_iter();
         if (prevTreeIter)
@@ -1055,6 +1056,16 @@ bool CtMainWin::file_open(const fs::path& filepath, const std::string& node_to_f
         _ctTextview.grab_focus();
     }
 
+    if (iterDocsRestore != _pCtConfig->recentDocsRestore.end())
+    {
+        auto nodes = CtStrUtil::gstring_split_to_int64(iterDocsRestore->second.visited_nodes.c_str(), ",");
+        if (nodes.size() > 0) {
+            _ctStateMachine.set_visited_nodes_list(nodes);
+            window_header_update();
+        }
+    }
+
+
     _pCtConfig->recentDocsFilepaths.move_or_push_front(fs::canonical(filepath));
     menu_set_items_recent_documents();
 
@@ -1113,7 +1124,7 @@ void CtMainWin::file_save(bool need_vacuum)
     if (_uCtStorage->save(need_vacuum, error))
     {
         update_window_save_not_needed();
-        get_state_machine().update_state();
+        _ctStateMachine.update_state();
     }
     else
     {
@@ -1133,6 +1144,7 @@ void CtMainWin::file_save_as(const std::string& new_filepath, const Glib::ustrin
 
     // remember expanded nodes for new file
     CtRecentDocRestore doc_state_restore;
+    doc_state_restore.visited_nodes = str::join_numbers(_ctStateMachine.get_visited_nodes_list(), ",");
     doc_state_restore.exp_coll_str = _uCtTreestore->treeview_get_tree_expanded_collapsed_string(*_uCtTreeview);
     if (const CtTreeIter curr_iter = curr_tree_iter())
     {
@@ -1231,7 +1243,7 @@ void CtMainWin::reset()
     auto on_scope_exit = scope_guard([&](void*) { user_active() = true; });
     user_active() = false;
 
-    get_state_machine().reset();
+    _ctStateMachine.reset();
 
     _uCtStorage.reset(CtStorageControl::create_dummy_storage(this));
 
@@ -1370,7 +1382,7 @@ void CtMainWin::update_window_save_needed(const CtSaveNeededUpdType update_type,
             rm_node_ids.push_back(top_node_id);
             _uCtTreestore->pending_rm_db_nodes(rm_node_ids);
             for (auto node_id: rm_node_ids)
-                get_state_machine().delete_states(node_id);
+                _ctStateMachine.delete_states(node_id);
         } break;
         case CtSaveNeededUpdType::book:
         {
@@ -1378,7 +1390,17 @@ void CtMainWin::update_window_save_needed(const CtSaveNeededUpdType update_type,
         } break;
     }
     if (new_machine_state && treeIter)
-        get_state_machine().update_state(treeIter);
+        _ctStateMachine.update_state(treeIter);
+}
+
+void CtMainWin::re_load_current_buffer(const bool new_machine_state)
+{
+    CtTreeIter currTreeIter = curr_tree_iter();
+    if (new_machine_state) {
+        _ctStateMachine.update_state(currTreeIter);
+    }
+    std::shared_ptr<CtNodeState> currState = _ctStateMachine.requested_state_current(currTreeIter.get_node_id());
+    load_buffer_from_state(currState, currTreeIter);
 }
 
 // Load Text Buffer from State Machine
@@ -1428,6 +1450,7 @@ void CtMainWin::load_buffer_from_state(std::shared_ptr<CtNodeState> state, CtTre
     _ctTextview.set_spell_check(curr_tree_iter().get_node_is_rich_text());
 
     text_buffer->place_cursor(text_buffer->get_iter_at_offset(state->cursor_pos));
+    (void)_try_move_focus_to_anchored_widget_if_on_it();
     Glib::signal_idle().connect_once([&](){
         auto insert_offset = _ctTextview.get_buffer()->get_insert()->get_iter().get_offset();
         if (_ctTextview.place_cursor_onscreen()) { // scroll if only cursor wasn't visible
@@ -1506,7 +1529,7 @@ void CtMainWin::_on_treeview_cursor_changed()
         {
             _fileSaveNeeded = true;
             rTextBuffer->set_modified(false);
-            get_state_machine().update_state(_prevTreeIter);
+            _ctStateMachine.update_state(_prevTreeIter);
         }
         _nodesCursorPos[prevNodeId] = rTextBuffer->property_cursor_position();
     }
@@ -1541,7 +1564,7 @@ void CtMainWin::_on_treeview_cursor_changed()
         update_selected_node_statusbar_info();
     }
 
-    get_state_machine().node_selected_changed(nodeId);
+    _ctStateMachine.node_selected_changed(nodeId);
 
     _prevTreeIter = treeIter;
 }
@@ -1797,16 +1820,16 @@ void CtMainWin::_on_textview_populate_popup(Gtk::Menu* menu)
                 if (do_set_cursor) curr_buffer()->place_cursor(target_iter);
             }
             //for (auto iter : menu->get_children()) menu->remove(*iter);
-            get_ct_menu().build_popup_menu(GTK_WIDGET(menu->gobj()), CtMenu::POPUP_MENU_TYPE::Link);
+            get_ct_menu().build_popup_menu(menu, CtMenu::POPUP_MENU_TYPE::Link);
         }
         else {
             //for (auto iter : menu->get_children()) menu->remove(*iter);
-            get_ct_menu().build_popup_menu(GTK_WIDGET(menu->gobj()), CtMenu::POPUP_MENU_TYPE::Text);
+            get_ct_menu().build_popup_menu(menu, CtMenu::POPUP_MENU_TYPE::Text);
         }
     }
     else {
         //for (auto iter : menu->get_children()) menu->remove(*iter);
-        _uCtActions->getCtMainWin()->get_ct_menu().build_popup_menu(GTK_WIDGET(menu->gobj()), CtMenu::POPUP_MENU_TYPE::Code);
+        _uCtActions->getCtMainWin()->get_ct_menu().build_popup_menu(menu, CtMenu::POPUP_MENU_TYPE::Code);
     }
 }
 
@@ -1859,6 +1882,23 @@ void CtMainWin::_on_textview_size_allocate(Gtk::Allocation& allocation)
     }
 }
 
+bool CtMainWin::_try_move_focus_to_anchored_widget_if_on_it()
+{
+    auto iter_insert = _ctTextview.get_buffer()->get_insert()->get_iter();
+    auto widgets = curr_tree_iter().get_embedded_pixbufs_tables_codeboxes(iter_insert.get_offset(), iter_insert.get_offset());
+    if (not widgets.empty()) {
+        if (CtCodebox* pCodebox = dynamic_cast<CtCodebox*>(widgets.front())) {
+            pCodebox->get_text_view().grab_focus();
+            return true;
+        }
+        if (CtTable* pTable = dynamic_cast<CtTable*>(widgets.front())) {
+            pTable->get_table_matrix().at(pTable->current_row()).at(pTable->current_column())->get_text_view().grab_focus();
+            return true;
+        }
+    }
+    return false;
+}
+
 bool CtMainWin::_on_textview_event(GdkEvent* event)
 {
     if (event->type != GDK_KEY_PRESS)
@@ -1880,23 +1920,10 @@ bool CtMainWin::_on_textview_event(GdkEvent* event)
     }
     else if (event->key.state & Gdk::CONTROL_MASK and event->key.keyval == GDK_KEY_space)
     {
-        auto iter_insert = curr_buffer->get_insert()->get_iter();
-        auto widgets = curr_tree_iter().get_embedded_pixbufs_tables_codeboxes(iter_insert.get_offset(), iter_insert.get_offset());
-        if (not widgets.empty()) {
-            if (CtCodebox* codebox = dynamic_cast<CtCodebox*>(widgets.front()))
-            {
-                codebox->get_text_view().grab_focus();
-                return true;
-            }
-            if (CtTable* table = dynamic_cast<CtTable*>(widgets.front()))
-            {
-                const CtTableMatrix& tableMatrix = table->get_table_matrix();
-                if (tableMatrix.size() and tableMatrix.front().size()) {
-                    tableMatrix.front().front()->get_text_view().grab_focus();
-                }
-                return true;
-            }
+        if (_try_move_focus_to_anchored_widget_if_on_it()) {
+            return true;
         }
+        auto iter_insert = _ctTextview.get_buffer()->get_insert()->get_iter();
         CtListInfo list_info = CtList(this, curr_buffer).get_paragraph_list_info(iter_insert);
         if (list_info and list_info.type == CtListType::Todo)
             if (_uCtActions->_is_curr_node_not_read_only_or_error())
@@ -2011,7 +2038,7 @@ void CtMainWin::_on_textview_event_after(GdkEvent* event)
     {
         if (curr_tree_iter() and not curr_buffer()->get_modified())
         {
-            get_state_machine().update_curr_state_cursor_pos(curr_tree_iter().get_node_id());
+            _ctStateMachine.update_curr_state_cursor_pos(curr_tree_iter().get_node_id());
         }
         if (event->type == GDK_BUTTON_PRESS)
         {
